/**************************************************************************//**
 * @file
 * @brief LCD Demo for EFM32_G8xx_DK, Energy Mode 0 (polled)
 * @author Energy Micro AS
 * @version 1.1.1
 ******************************************************************************
 * @section License
 * <b>(C) Copyright 2010 Energy Micro AS, http://www.energymicro.com</b>
 ******************************************************************************
 *
 * This source code is the property of Energy Micro AS. The source and compiled
 * code may only be used on Energy Micro "EFM32" microcontrollers.
 *
 * This copyright notice may not be removed from the source code nor changed.
 *
 * DISCLAIMER OF WARRANTY/LIMITATION OF REMEDIES: Energy Micro AS has no
 * obligation to support this Software. Energy Micro AS is providing the
 * Software "AS IS", with no express or implied warranties of any kind,
 * including, but not limited to, any implied warranties of merchantability
 * or fitness for any particular purpose or warranties against infringement
 * of any proprietary rights of a third party.
 *
 * Energy Micro AS will not be liable for any consequential, incidental, or
 * special damages, or any other relief, or for any claim by any third party,
 * arising from your use of this Software.
 *
 *****************************************************************************/
#include <stddef.h>
#include "chip.h"
#include "efm32.h"
#include "lcdcontroller.h"
#include "lcdtest.h"
#include "rtc.h"
#include "vcmp.h"

volatile uint32_t msTicks; /* counts 1ms timeTicks */

void AdditionalConfig(void);
void BuzzerCtrl(void);
void ClockUpdate(void);

/**************************************************************************//**
 * @brief SysTick_Handler
 * Interrupt Service Routine for system tick counter
 *****************************************************************************/
void SysTick_Handler(void)
{
  msTicks++;       /* increment counter necessary in Delay()*/
  BuzzerCtrl();
}

/**************************************************************************//**
 * @brief Delays number of msTick Systicks (typically 1 ms)
 * @param dlyTicks Number of ticks to delay
 *****************************************************************************/
void Delay(uint32_t dlyTicks)
{
  uint32_t curTicks;

  curTicks = msTicks;
  while ((msTicks - curTicks) < dlyTicks) ;
}

/**************************************************************************//**
 * @brief  Main function
 *****************************************************************************/
int main(void)
{
	/* Chip errata */
	CHIP_init();
	
	// turn on clocking of all the modules
	// *NOTE: THIS IS A COMPATABILITY FIX, previous releases had all the modules clocked at power on
	CMU_TypeDef *cmu = CMU;
	cmu->HFCORECLKEN0 |= 0x0F;
	cmu->HFPERCLKEN0 |= 0xFFFF;
	
	// configure oscillator
	// enable LFRC0 oscillator
//	cmu->OSCENCMD     |= CMU_OSCENCMD_LFRCOEN;
//	while (!(cmu->STATUS & CMU_STATUS_LFRCORDY)) ;
	cmu->OSCENCMD     |= CMU_OSCENCMD_LFXOEN;
	while (!(cmu->STATUS & CMU_STATUS_LFXORDY)) ;

	// select 1MHz for HFRC0
	cmu->HFRCOCTRL &= ~(_CMU_HFRCOCTRL_BAND_MASK);
	cmu->HFRCOCTRL |= CMU_HFRCOCTRL_BAND_1MHZ;
	
	cmu->HFCORECLKDIV = CMU_HFCORECLKDIV_HFCORECLKDIV_HFCLK4;
	cmu->HFPERCLKDIV =	CMU_HFPERCLKDIV_HFPERCLKDIV_HFCLK4 |
					CMU_HFPERCLKDIV_HFPERCLKEN;

	
	/* Ensure core frequency has been updated */
	SystemCoreClockUpdate();
	/* Setup SysTick Timer for 1 msec interrupts  */
//	if (SysTick_Config(SystemCoreClock / 1000)) while (1) ;
	if (SysTick_Config(SystemCoreClock / 3000)) while (1) ;
	
	/* Initialize necessary clocks and controller */
	LCD_Init(LCD);
	RTC_init();
	RTC_trigger(500, ClockUpdate);
	VCMP_Init();
	AdditionalConfig();
	
	/* Run demo */
	LCD_Test(LCD);
}

void AdditionalConfig(void)
{
	GPIO_TypeDef *gpio = GPIO;
	
	/* Configure PE0, PE1, PE2, PE3 as input with pull up */
	gpio->P[4].MODEL &= ~(_GPIO_P_MODEL_MODE0_MASK);
	gpio->P[4].MODEL |= GPIO_P_MODEL_MODE0_INPUTPULLFILTER;
	gpio->P[4].DOUT |= (1 << 0);

	gpio->P[4].MODEL &= ~(_GPIO_P_MODEL_MODE1_MASK);
	gpio->P[4].MODEL |= GPIO_P_MODEL_MODE1_INPUTPULLFILTER;
	gpio->P[4].DOUT |= (1 << 1);

	gpio->P[4].MODEL &= ~(_GPIO_P_MODEL_MODE2_MASK);
	gpio->P[4].MODEL |= GPIO_P_MODEL_MODE2_INPUTPULLFILTER;
	gpio->P[4].DOUT |= (1 << 2);

	gpio->P[4].MODEL &= ~(_GPIO_P_MODEL_MODE3_MASK);
	gpio->P[4].MODEL |= GPIO_P_MODEL_MODE3_INPUTPULLFILTER;
	gpio->P[4].DOUT |= (1 << 3);
	
	// enable buzzer PA12, PA13
	gpio->P[0].MODEH &= ~(_GPIO_P_MODEH_MODE12_MASK);
	gpio->P[0].MODEH |= GPIO_P_MODEH_MODE12_PUSHPULLDRIVE;
	gpio->P[0].DOUT &= ~(1 << 12);

	gpio->P[0].MODEH &= ~(_GPIO_P_MODEH_MODE13_MASK);
	gpio->P[0].MODEH |= GPIO_P_MODEH_MODE13_PUSHPULLDRIVE;
	gpio->P[0].DOUT &= ~(1 << 13);

	gpio->P[0].CTRL = 0x2; // 20mA outputs
	
	// turn off unnecessary modules
	CMU_TypeDef *cmu = CMU;
	
	cmu->HFCORECLKEN0 &= ~(	CMU_HFCORECLKEN0_AES|
						CMU_HFCORECLKEN0_DMA|
						CMU_HFCORECLKEN0_EBI );
	
	cmu->HFPERCLKEN0 &= ~(	CMU_HFPERCLKEN0_USART0|
						CMU_HFPERCLKEN0_USART1|
						CMU_HFPERCLKEN0_USART2|
						CMU_HFPERCLKEN0_UART0|
						CMU_HFPERCLKEN0_ACMP0|
						CMU_HFPERCLKEN0_ACMP1|
						CMU_HFPERCLKEN0_PRS|
						CMU_HFPERCLKEN0_DAC0|
						CMU_HFPERCLKEN0_ADC0|
						CMU_HFPERCLKEN0_I2C0 );
}